////////////////////////////////////////////////////////
// 
// code adapted from Paul Bourke www.paulbourke.net
// 
//
////////////////////////////////////////////////////////

void fft_1D(int dir, int n, float[] ir, float[] im) {

  // m is the two-logarithm of n
  float mt = (log(n)/log(2));
  int m = (int)mt;
  if (mt-m>1e-4) {
    println("Buffer size must be power of two"); 
    exit();
  }
  /* Do the bit reversal */
  int i2 = n >> 1;
  int j = 0;
  for (int i=0; i<n-1; i++) {
    if (i < j) {
      // swap [i] and [j]
      float tx = ir[i]; 
      ir[i] = ir[j]; 
      ir[j] = tx;
      float ty = im[i]; 
      im[i] = im[j]; 
      im[j] = ty;
    }
    int k = i2;
    while (k <= j) {
      j -= k;
      k >>= 1;
    }
    j += k;
  }

  /* Compute the FFT */
  float c1 = -1.0; 
  float c2 = 0.0;
  int l2 = 1;
  for (int l=0; l<m; l++) {
    int l1 = l2;
    l2 <<= 1;
    float u1 = 1.0; 
    float u2 = 0.0;
    for (j=0; j<l1; j++) {
      for (int i=j; i<n; i+=l2) {
        int i1 = i + l1;
        float t1 = u1 * ir[i1] - u2 * im[i1];
        float t2 = u1 * im[i1] + u2 * ir[i1];
        ir[i1] = ir[i] - t1; 
        im[i1] = im[i] - t2;
        ir[i] += t1;
        im[i] += t2;
      }
      float z =  u1 * c1 - u2 * c2;
      u2 = u1 * c2 + u2 * c1;
      u1 = z;
    }
    c2 = sqrt((1.0 - c1) / 2.0);
    if (dir == 1) c2 = -c2;
    c1 = sqrt((1.0 + c1) / 2.0);
  }

  float sn = sqrt(n);
  for (int i=0; i<n; i++) {
    ir[i] /= sn;
    im[i] /= sn;
  }

  return;
}